//
//  SITLocationInterface.h
//  SitumSDK
//
//  Created by A Barros on 27/2/17.
//  Copyright © 2017 Situm. All rights reserved.
//

#ifndef SITLocationInterface_h
#define SITLocationInterface_h

#import "SITLocationRequest.h"
#import "SITLocation.h"
#import "SITLocationParametersUpdate.h"
#import "SITGeofence.h"

/**
 *  SITLocationState
 *
 *  @note: Represents the current state of the SITLocationManager.
 */
typedef NS_ENUM(int, SITLocationState) {
    /**
     *  Location is not running.
     */
            kSITLocationStopped = 0,

    /*
     *  System is still calculating.
     */
            kSITLocationCalculating,

    /*
     *  System cannot determine where the user is in that building.
     */
            kSITLocationUserNotInBuilding,

    /**
     *  Location is running.
     */
            kSITLocationStarted,
    
    /**
     *  Compass needs to be calibrated
     */
            kSITLocationCompassNeedsCalibration,
};


/**
 General interface every object providing location should conform to.
 */
@protocol SITLocationInterface <NSObject>

/**
 Provides APIs for requesting users current location.
 
Calling this method several times in a state different to kSITLocationStopped, would have different behaviours depending on the request being passed:

 If the request is equal to the previously provided wont do anything.

 If the request is different would cause a restart, stopping location updates and starting it again with the new request.
 Starting from version 2.52.0 a nil request can be passed to this method. This means a remote configuration will be used to configure the location engine.
 @see To learn more about  remote configuration and how to configure each parameter go to <https://situm.com/docs/07-remote-configuration/>

 @param request SITLocationRequest object
 */
- (void)requestLocationUpdates:(SITLocationRequest *_Nullable)request;

/**
 State of the shared instance.

 @return SITLocationState value that indicates the actual state of the system
 */
- (SITLocationState)state;

/**
 Stops the delivery of location updates.
 
 @note This method has no efect if the system is not running.
 */
- (void)removeUpdates;

/**
 Updates parameters of the positioning system.
 The purpose of this call is to modify parameters of the location system on the fly, when it has already been started.
 It does nothing if the service is stopped. In that case location system should be started with a call to {@link #requestLocationUpdates:}.
 
 @param update object that contains parameters to modify the location system on the fly.
 @since 2.42.0
 */
- (void)updateLocationParameters:(SITLocationParametersUpdate * _Nonnull)update;

@end


/**
 Delegate for SITLocationManager. Use this to listen for location related callbacks.
 */
@protocol SITLocationDelegate <NSObject>

#pragma mark - location related events


/**
 Called when user’s location changes.

 @param locationManager location provider
 @param location current location
 */
- (void)locationManager:(id <SITLocationInterface> _Nonnull)locationManager
      didUpdateLocation:(SITLocation * _Nonnull)location;


/**
 Called when an error is encountered.

 @param locationManager location provider
 @param error error description
 @note Invoked when an error has occurred. Error types are defined in "SITLocationError.h".
 */
- (void)locationManager:(id <SITLocationInterface> _Nonnull)locationManager
       didFailWithError:(NSError * _Nullable)error;

/**
 Called when the system changes it's internal state

 @param locationManager location provider
 @param state state of the system (one of SITLocationState enum values)
 */
- (void)locationManager:(id <SITLocationInterface> _Nonnull)locationManager
         didUpdateState:(SITLocationState)state;


@optional
/**
 Called every time the location engine is initialized with a valid `SITLocationRequest`. Optional callback that allows the developer to know the parameters that are being used to initialize the location engine. Specially important when using Remote Configuration in your app.
 
 @param locationManager location provider
 @param request `SITLocationRequest` instance
 */
- (void)locationManager:(id <SITLocationInterface> _Nonnull)locationManager
didInitiatedWithRequest:(SITLocationRequest * _Nonnull)request;
@end


/**
 Delegate for SITLocationManager. Use this to listen for geofences related callbacks.
 */
@protocol SITGeofencesDelegate <NSObject>

#pragma mark - geofences related events
/**
 Called every time the user enter in one or more of the geofences defined in the dashboard.
 
 @param geofences the geofences where the user has entered
 */
-(void)didEnteredGeofences:(NSArray <SITGeofence *> *)geofences;

/**
 Called every time the user exited one or more geofences. A geofence is only added to the list if in the previous location update the user was inside it and in the last location update he is out.
 
 @param geofences the geofences where the user has entered
 */
-(void)didExitedGeofences:(NSArray <SITGeofence *> *)geofences;

@end

#endif /* SITLocationInterface_h */
